package com.company.unfetchedattribute2.screen.invoice

import com.company.unfetchedattribute2.entity.Company
import com.company.unfetchedattribute2.entity.Invoice
import com.company.unfetchedattribute2.entity.InvoiceEntry
import com.company.unfetchedattribute2.entity.TaxData
import io.jmix.core.DataManager
import io.jmix.core.EntityStates
import io.jmix.core.Metadata
import io.jmix.ui.Actions
import io.jmix.ui.Notifications
import io.jmix.ui.ScreenBuilders
import io.jmix.ui.UiComponents
import io.jmix.ui.action.Action
import io.jmix.ui.action.entitypicker.EntityClearAction
import io.jmix.ui.action.entitypicker.EntityLookupAction
import io.jmix.ui.component.*
import io.jmix.ui.component.data.options.ContainerOptions
import io.jmix.ui.component.data.value.ContainerValueSource
import io.jmix.ui.model.CollectionContainer
import io.jmix.ui.model.CollectionPropertyContainer
import io.jmix.ui.screen.*
import org.springframework.beans.factory.annotation.Autowired

@UiController("Invoice.edit")
@UiDescriptor("invoice-edit.xml")
@EditedEntityContainer("invoiceDc")
class InvoiceEdit : StandardEditor<Invoice>() {
    @Autowired
    private lateinit var notifications: Notifications

    @Autowired
    private lateinit var entriesDc: CollectionPropertyContainer<InvoiceEntry>

    @Autowired
    private lateinit var dataManager: DataManager

    @Autowired
    private lateinit var uiComponents: UiComponents

    @Autowired
    private lateinit var taxesDc: CollectionContainer<TaxData>

    @Autowired
    private lateinit var actions: Actions

    @Autowired
    private lateinit var entriesTable: Table<InvoiceEntry>

    @Autowired
    private lateinit var screenBuilders: ScreenBuilders

    @Autowired
    private lateinit var entityStates: EntityStates

    @Autowired
    private lateinit var companyField: EntityPicker<Company>

    @Autowired
    private lateinit var metadata: Metadata

    @Subscribe
    private fun onAfterShow(event: AfterShowEvent) {
        if (entityStates.isNew(editedEntity)) {
            screenBuilders
                .lookup(companyField)
                .withOpenMode(OpenMode.DIALOG)
                .withSelectValidator { it.selectedItems.size == 1 }
                .build()
                .show()
        }
        if (editedEntity.entries.size == 0) {
            metadata.create(InvoiceEntry::class.java).also {
                it.name = "test"
                it.invoice = editedEntity
                entriesDc.mutableItems.add(it)
            }
        }
    }

    @Subscribe
    private fun onInitEntity(event: InitEntityEvent<Invoice>) {
//        val invoice = editedEntity
//        var entries: MutableList<InvoiceEntry>? = invoice.entries
//        if (entries == null) {
//            entries = ArrayList()
//        }
//        invoice.entries = entries


//        val entry = screenData.dataContext.create(InvoiceEntry::class.java).also {
//            it.invoice = editedEntity
//            editedEntity.entries.add(it)
//            screenData.dataContext.merge(it)
//        }
//        entriesDc.setItems(editedEntity.entries)
    }

    @Subscribe("companyField")
    private fun onCompanyFieldValueChange(event: HasValue.ValueChangeEvent<Company>) {
        editedEntity.entries.firstOrNull()?.let {
            it.taxData = event.value?.defaultTaxData
        }
    }

    @Subscribe("checkTaxInformationBtn")
    private fun onCheckTaxInformationBtnClick(event: Button.ClickEvent) {
        editedEntity.entries.forEach {
            println("test: " + it.taxData?.taxInformation)
        }
        notifications.create()
            .withCaption("all ok")
            .show()
    }

    @Subscribe("entriesTable.create")
    private fun onEntriesTableCreate(event: Action.ActionPerformedEvent) {
        val invoiceEntry = screenData.dataContext.merge(
            dataManager.create(InvoiceEntry::class.java)
        )
        invoiceEntry.invoice = editedEntity
        entriesDc.replaceItem(invoiceEntry)
    }

    @Install(to = "entriesTable.taxData", subject = "columnGenerator")
    private fun entriesTableTaxDataColumnGenerator(invoiceEntry: InvoiceEntry): Component {
        return uiComponents.create(EntityComboBox::class.java).also { field ->
            field.setWidthFull()
            field.addAction(actions.create(EntityLookupAction.ID))
            field.addAction(actions.create(EntityClearAction.ID))

            field.options = ContainerOptions(taxesDc)
            field.nullSelectionCaption = "no tax"
            field.valueSource = ContainerValueSource(entriesTable.getInstanceContainer(invoiceEntry), "taxData")


        }
    }

}